package dev.micle.totemofreviving.network;

import dev.micle.totemofreviving.TotemOfReviving;
import dev.micle.totemofreviving.item.totem.TotemItem;
import net.minecraft.ChatFormatting;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.network.NetworkEvent;

import java.util.function.Supplier;

public class ReviveTargetPacket {
	private final InteractionHand hand;
	
	public ReviveTargetPacket(final InteractionHand hand) {
		this.hand = hand;
	}
	
	public static void encode(final ReviveTargetPacket packet, final FriendlyByteBuf buffer) {
		Network.writeVersionInfo(buffer, false);
		buffer.writeEnum(packet.hand);
	}
	
	public static ReviveTargetPacket decode(final FriendlyByteBuf buffer) {
		Network.checkVersion(buffer);
		return new ReviveTargetPacket(buffer.readEnum(InteractionHand.class));
	}
	
	public static void handle(final ReviveTargetPacket packet, final Supplier<NetworkEvent.Context> contextSupplier) {
		final NetworkEvent.Context context = contextSupplier.get();
		context.enqueueWork(() -> {
			ServerPlayer sender = context.getSender();
			if (sender == null) { return; }
			
			ItemStack totemStack = sender.getItemInHand(packet.hand);
			ServerPlayer target;
			
			try {
				target = TotemOfReviving.PROXY.getServer().getPlayerList().getPlayer(TotemItem.getTargetUUID(totemStack));
				if (target == null) { throw new NullPointerException("Target is null!"); }
			} catch (NullPointerException exception) {
				sender.sendMessage(new TextComponent(ChatFormatting.WHITE + "Unable to find player!"), sender.getUUID());
				return;
			}
			
			if (!target.isSpectator()) {
				sender.sendMessage(new TextComponent(ChatFormatting.GRAY + target.getDisplayName().getString() + ChatFormatting.WHITE + " is not dead!"), sender.getUUID());
				return;
			}
			
			if (!target.getLevel().equals(sender.getLevel()) && !TotemItem.canReviveAcrossDimensions(totemStack)) {
				sender.sendMessage(new TextComponent(ChatFormatting.GRAY + target.getDisplayName().getString() + ChatFormatting.WHITE + " is in a different dimension!"), sender.getUUID());
				return;
			}
			
			TotemItem.setTargetDeaths(totemStack, target);
			if (!TotemItem.canTotemAffordTarget(totemStack)) {
				sender.sendMessage(new TextComponent(ChatFormatting.WHITE + "Not enough charge!"), sender.getUUID());
				return;
			}
			
			target.teleportTo(sender.getLevel(), sender.position().x, sender.position().y, sender.position().z, sender.getYRot(), sender.getXRot());
			target.setGameMode(sender.getLevel().getServer().getDefaultGameType());
			TotemItem.setCharge(totemStack, TotemItem.getCharge(totemStack) - TotemItem.getTargetCost(totemStack));
			totemStack.hurtAndBreak(1, sender, e -> e.broadcastBreakEvent(packet.hand));
			sender.sendMessage(new TextComponent(ChatFormatting.WHITE + "Successfully revived " + ChatFormatting.GRAY + target.getDisplayName().getString() + "!"), sender.getUUID());
		});
		context.setPacketHandled(true);
	}
}
